@@ -231,11 +231,7 @@ static char *(features[]) =
 #else
 	"-dnd",
 #endif
-#ifdef EBCDIC
-	"+ebcdic",
-#else
 	"-ebcdic",
-#endif
 #ifdef FEAT_EMACS_TAGS
 	"+emacs_tags",
 #else
@@ -312,11 +308,7 @@ static char *(features[]) =
 #else
 	"-job",
 #endif
-#ifdef FEAT_JUMPLIST
 	"+jumplist",
-#else
-	"-jumplist",
-#endif
 #ifdef FEAT_KEYMAP
 	"+keymap",
 #else
@@ -556,7 +548,11 @@ static char *(features[]) =
 	"-smartindent",
 #endif
 #ifdef FEAT_SODIUM
+# ifdef DYNAMIC_SODIUM
+	"+sodium/dyn",
+# else
 	"+sodium",
+# endif
 #else
 	"-sodium",
 #endif
@@ -665,14 +661,15 @@ static char *(features[]) =
 	"-vartabs",
 #endif
 	"+vertsplit",
-	"+virtualedit",
-	"+visual",
-	"+visualextra",
+	"+vim9script",
 #ifdef FEAT_VIMINFO
 	"+viminfo",
 #else
 	"-viminfo",
 #endif
+	"+virtualedit",
+	"+visual",
+	"+visualextra",
 	"+vreplace",
 #ifdef MSWIN
 # ifdef FEAT_VTP
@@ -753,6 +750,1240 @@ static char *(features[]) =
 
 static int included_patches[] =
 {   /* Add new patch number below this line */
+/**/
+    4372,
+/**/
+    4371,
+/**/
+    4370,
+/**/
+    4369,
+/**/
+    4368,
+/**/
+    4367,
+/**/
+    4366,
+/**/
+    4365,
+/**/
+    4364,
+/**/
+    4363,
+/**/
+    4362,
+/**/
+    4361,
+/**/
+    4360,
+/**/
+    4359,
+/**/
+    4358,
+/**/
+    4357,
+/**/
+    4356,
+/**/
+    4355,
+/**/
+    4354,
+/**/
+    4353,
+/**/
+    4352,
+/**/
+    4351,
+/**/
+    4350,
+/**/
+    4349,
+/**/
+    4348,
+/**/
+    4347,
+/**/
+    4346,
+/**/
+    4345,
+/**/
+    4344,
+/**/
+    4343,
+/**/
+    4342,
+/**/
+    4341,
+/**/
+    4340,
+/**/
+    4339,
+/**/
+    4338,
+/**/
+    4337,
+/**/
+    4336,
+/**/
+    4335,
+/**/
+    4334,
+/**/
+    4333,
+/**/
+    4332,
+/**/
+    4331,
+/**/
+    4330,
+/**/
+    4329,
+/**/
+    4328,
+/**/
+    4327,
+/**/
+    4326,
+/**/
+    4325,
+/**/
+    4324,
+/**/
+    4323,
+/**/
+    4322,
+/**/
+    4321,
+/**/
+    4320,
+/**/
+    4319,
+/**/
+    4318,
+/**/
+    4317,
+/**/
+    4316,
+/**/
+    4315,
+/**/
+    4314,
+/**/
+    4313,
+/**/
+    4312,
+/**/
+    4311,
+/**/
+    4310,
+/**/
+    4309,
+/**/
+    4308,
+/**/
+    4307,
+/**/
+    4306,
+/**/
+    4305,
+/**/
+    4304,
+/**/
+    4303,
+/**/
+    4302,
+/**/
+    4301,
+/**/
+    4300,
+/**/
+    4299,
+/**/
+    4298,
+/**/
+    4297,
+/**/
+    4296,
+/**/
+    4295,
+/**/
+    4294,
+/**/
+    4293,
+/**/
+    4292,
+/**/
+    4291,
+/**/
+    4290,
+/**/
+    4289,
+/**/
+    4288,
+/**/
+    4287,
+/**/
+    4286,
+/**/
+    4285,
+/**/
+    4284,
+/**/
+    4283,
+/**/
+    4282,
+/**/
+    4281,
+/**/
+    4280,
+/**/
+    4279,
+/**/
+    4278,
+/**/
+    4277,
+/**/
+    4276,
+/**/
+    4275,
+/**/
+    4274,
+/**/
+    4273,
+/**/
+    4272,
+/**/
+    4271,
+/**/
+    4270,
+/**/
+    4269,
+/**/
+    4268,
+/**/
+    4267,
+/**/
+    4266,
+/**/
+    4265,
+/**/
+    4264,
+/**/
+    4263,
+/**/
+    4262,
+/**/
+    4261,
+/**/
+    4260,
+/**/
+    4259,
+/**/
+    4258,
+/**/
+    4257,
+/**/
+    4256,
+/**/
+    4255,
+/**/
+    4254,
+/**/
+    4253,
+/**/
+    4252,
+/**/
+    4251,
+/**/
+    4250,
+/**/
+    4249,
+/**/
+    4248,
+/**/
+    4247,
+/**/
+    4246,
+/**/
+    4245,
+/**/
+    4244,
+/**/
+    4243,
+/**/
+    4242,
+/**/
+    4241,
+/**/
+    4240,
+/**/
+    4239,
+/**/
+    4238,
+/**/
+    4237,
+/**/
+    4236,
+/**/
+    4235,
+/**/
+    4234,
+/**/
+    4233,
+/**/
+    4232,
+/**/
+    4231,
+/**/
+    4230,
+/**/
+    4229,
+/**/
+    4228,
+/**/
+    4227,
+/**/
+    4226,
+/**/
+    4225,
+/**/
+    4224,
+/**/
+    4223,
+/**/
+    4222,
+/**/
+    4221,
+/**/
+    4220,
+/**/
+    4219,
+/**/
+    4218,
+/**/
+    4217,
+/**/
+    4216,
+/**/
+    4215,
+/**/
+    4214,
+/**/
+    4213,
+/**/
+    4212,
+/**/
+    4211,
+/**/
+    4210,
+/**/
+    4209,
+/**/
+    4208,
+/**/
+    4207,
+/**/
+    4206,
+/**/
+    4205,
+/**/
+    4204,
+/**/
+    4203,
+/**/
+    4202,
+/**/
+    4201,
+/**/
+    4200,
+/**/
+    4199,
+/**/
+    4198,
+/**/
+    4197,
+/**/
+    4196,
+/**/
+    4195,
+/**/
+    4194,
+/**/
+    4193,
+/**/
+    4192,
+/**/
+    4191,
+/**/
+    4190,
+/**/
+    4189,
+/**/
+    4188,
+/**/
+    4187,
+/**/
+    4186,
+/**/
+    4185,
+/**/
+    4184,
+/**/
+    4183,
+/**/
+    4182,
+/**/
+    4181,
+/**/
+    4180,
+/**/
+    4179,
+/**/
+    4178,
+/**/
+    4177,
+/**/
+    4176,
+/**/
+    4175,
+/**/
+    4174,
+/**/
+    4173,
+/**/
+    4172,
+/**/
+    4171,
+/**/
+    4170,
+/**/
+    4169,
+/**/
+    4168,
+/**/
+    4167,
+/**/
+    4166,
+/**/
+    4165,
+/**/
+    4164,
+/**/
+    4163,
+/**/
+    4162,
+/**/
+    4161,
+/**/
+    4160,
+/**/
+    4159,
+/**/
+    4158,
+/**/
+    4157,
+/**/
+    4156,
+/**/
+    4155,
+/**/
+    4154,
+/**/
+    4153,
+/**/
+    4152,
+/**/
+    4151,
+/**/
+    4150,
+/**/
+    4149,
+/**/
+    4148,
+/**/
+    4147,
+/**/
+    4146,
+/**/
+    4145,
+/**/
+    4144,
+/**/
+    4143,
+/**/
+    4142,
+/**/
+    4141,
+/**/
+    4140,
+/**/
+    4139,
+/**/
+    4138,
+/**/
+    4137,
+/**/
+    4136,
+/**/
+    4135,
+/**/
+    4134,
+/**/
+    4133,
+/**/
+    4132,
+/**/
+    4131,
+/**/
+    4130,
+/**/
+    4129,
+/**/
+    4128,
+/**/
+    4127,
+/**/
+    4126,
+/**/
+    4125,
+/**/
+    4124,
+/**/
+    4123,
+/**/
+    4122,
+/**/
+    4121,
+/**/
+    4120,
+/**/
+    4119,
+/**/
+    4118,
+/**/
+    4117,
+/**/
+    4116,
+/**/
+    4115,
+/**/
+    4114,
+/**/
+    4113,
+/**/
+    4112,
+/**/
+    4111,
+/**/
+    4110,
+/**/
+    4109,
+/**/
+    4108,
+/**/
+    4107,
+/**/
+    4106,
+/**/
+    4105,
+/**/
+    4104,
+/**/
+    4103,
+/**/
+    4102,
+/**/
+    4101,
+/**/
+    4100,
+/**/
+    4099,
+/**/
+    4098,
+/**/
+    4097,
+/**/
+    4096,
+/**/
+    4095,
+/**/
+    4094,
+/**/
+    4093,
+/**/
+    4092,
+/**/
+    4091,
+/**/
+    4090,
+/**/
+    4089,
+/**/
+    4088,
+/**/
+    4087,
+/**/
+    4086,
+/**/
+    4085,
+/**/
+    4084,
+/**/
+    4083,
+/**/
+    4082,
+/**/
+    4081,
+/**/
+    4080,
+/**/
+    4079,
+/**/
+    4078,
+/**/
+    4077,
+/**/
+    4076,
+/**/
+    4075,
+/**/
+    4074,
+/**/
+    4073,
+/**/
+    4072,
+/**/
+    4071,
+/**/
+    4070,
+/**/
+    4069,
+/**/
+    4068,
+/**/
+    4067,
+/**/
+    4066,
+/**/
+    4065,
+/**/
+    4064,
+/**/
+    4063,
+/**/
+    4062,
+/**/
+    4061,
+/**/
+    4060,
+/**/
+    4059,
+/**/
+    4058,
+/**/
+    4057,
+/**/
+    4056,
+/**/
+    4055,
+/**/
+    4054,
+/**/
+    4053,
+/**/
+    4052,
+/**/
+    4051,
+/**/
+    4050,
+/**/
+    4049,
+/**/
+    4048,
+/**/
+    4047,
+/**/
+    4046,
+/**/
+    4045,
+/**/
+    4044,
+/**/
+    4043,
+/**/
+    4042,
+/**/
+    4041,
+/**/
+    4040,
+/**/
+    4039,
+/**/
+    4038,
+/**/
+    4037,
+/**/
+    4036,
+/**/
+    4035,
+/**/
+    4034,
+/**/
+    4033,
+/**/
+    4032,
+/**/
+    4031,
+/**/
+    4030,
+/**/
+    4029,
+/**/
+    4028,
+/**/
+    4027,
+/**/
+    4026,
+/**/
+    4025,
+/**/
+    4024,
+/**/
+    4023,
+/**/
+    4022,
+/**/
+    4021,
+/**/
+    4020,
+/**/
+    4019,
+/**/
+    4018,
+/**/
+    4017,
+/**/
+    4016,
+/**/
+    4015,
+/**/
+    4014,
+/**/
+    4013,
+/**/
+    4012,
+/**/
+    4011,
+/**/
+    4010,
+/**/
+    4009,
+/**/
+    4008,
+/**/
+    4007,
+/**/
+    4006,
+/**/
+    4005,
+/**/
+    4004,
+/**/
+    4003,
+/**/
+    4002,
+/**/
+    4001,
+/**/
+    4000,
+/**/
+    3999,
+/**/
+    3998,
+/**/
+    3997,
+/**/
+    3996,
+/**/
+    3995,
+/**/
+    3994,
+/**/
+    3993,
+/**/
+    3992,
+/**/
+    3991,
+/**/
+    3990,
+/**/
+    3989,
+/**/
+    3988,
+/**/
+    3987,
+/**/
+    3986,
+/**/
+    3985,
+/**/
+    3984,
+/**/
+    3983,
+/**/
+    3982,
+/**/
+    3981,
+/**/
+    3980,
+/**/
+    3979,
+/**/
+    3978,
+/**/
+    3977,
+/**/
+    3976,
+/**/
+    3975,
+/**/
+    3974,
+/**/
+    3973,
+/**/
+    3972,
+/**/
+    3971,
+/**/
+    3970,
+/**/
+    3969,
+/**/
+    3968,
+/**/
+    3967,
+/**/
+    3966,
+/**/
+    3965,
+/**/
+    3964,
+/**/
+    3963,
+/**/
+    3962,
+/**/
+    3961,
+/**/
+    3960,
+/**/
+    3959,
+/**/
+    3958,
+/**/
+    3957,
+/**/
+    3956,
+/**/
+    3955,
+/**/
+    3954,
+/**/
+    3953,
+/**/
+    3952,
+/**/
+    3951,
+/**/
+    3950,
+/**/
+    3949,
+/**/
+    3948,
+/**/
+    3947,
+/**/
+    3946,
+/**/
+    3945,
+/**/
+    3944,
+/**/
+    3943,
+/**/
+    3942,
+/**/
+    3941,
+/**/
+    3940,
+/**/
+    3939,
+/**/
+    3938,
+/**/
+    3937,
+/**/
+    3936,
+/**/
+    3935,
+/**/
+    3934,
+/**/
+    3933,
+/**/
+    3932,
+/**/
+    3931,
+/**/
+    3930,
+/**/
+    3929,
+/**/
+    3928,
+/**/
+    3927,
+/**/
+    3926,
+/**/
+    3925,
+/**/
+    3924,
+/**/
+    3923,
+/**/
+    3922,
+/**/
+    3921,
+/**/
+    3920,
+/**/
+    3919,
+/**/
+    3918,
+/**/
+    3917,
+/**/
+    3916,
+/**/
+    3915,
+/**/
+    3914,
+/**/
+    3913,
+/**/
+    3912,
+/**/
+    3911,
+/**/
+    3910,
+/**/
+    3909,
+/**/
+    3908,
+/**/
+    3907,
+/**/
+    3906,
+/**/
+    3905,
+/**/
+    3904,
+/**/
+    3903,
+/**/
+    3902,
+/**/
+    3901,
+/**/
+    3900,
+/**/
+    3899,
+/**/
+    3898,
+/**/
+    3897,
+/**/
+    3896,
+/**/
+    3895,
+/**/
+    3894,
+/**/
+    3893,
+/**/
+    3892,
+/**/
+    3891,
+/**/
+    3890,
+/**/
+    3889,
+/**/
+    3888,
+/**/
+    3887,
+/**/
+    3886,
+/**/
+    3885,
+/**/
+    3884,
+/**/
+    3883,
+/**/
+    3882,
+/**/
+    3881,
+/**/
+    3880,
+/**/
+    3879,
+/**/
+    3878,
+/**/
+    3877,
+/**/
+    3876,
+/**/
+    3875,
+/**/
+    3874,
+/**/
+    3873,
+/**/
+    3872,
+/**/
+    3871,
+/**/
+    3870,
+/**/
+    3869,
+/**/
+    3868,
+/**/
+    3867,
+/**/
+    3866,
+/**/
+    3865,
+/**/
+    3864,
+/**/
+    3863,
+/**/
+    3862,
+/**/
+    3861,
+/**/
+    3860,
+/**/
+    3859,
+/**/
+    3858,
+/**/
+    3857,
+/**/
+    3856,
+/**/
+    3855,
+/**/
+    3854,
+/**/
+    3853,
+/**/
+    3852,
+/**/
+    3851,
+/**/
+    3850,
+/**/
+    3849,
+/**/
+    3848,
+/**/
+    3847,
+/**/
+    3846,
+/**/
+    3845,
+/**/
+    3844,
+/**/
+    3843,
+/**/
+    3842,
+/**/
+    3841,
+/**/
+    3840,
+/**/
+    3839,
+/**/
+    3838,
+/**/
+    3837,
+/**/
+    3836,
+/**/
+    3835,
+/**/
+    3834,
+/**/
+    3833,
+/**/
+    3832,
+/**/
+    3831,
+/**/
+    3830,
+/**/
+    3829,
+/**/
+    3828,
+/**/
+    3827,
+/**/
+    3826,
+/**/
+    3825,
+/**/
+    3824,
+/**/
+    3823,
+/**/
+    3822,
+/**/
+    3821,
+/**/
+    3820,
+/**/
+    3819,
+/**/
+    3818,
+/**/
+    3817,
+/**/
+    3816,
+/**/
+    3815,
+/**/
+    3814,
+/**/
+    3813,
+/**/
+    3812,
+/**/
+    3811,
+/**/
+    3810,
+/**/
+    3809,
+/**/
+    3808,
+/**/
+    3807,
+/**/
+    3806,
+/**/
+    3805,
+/**/
+    3804,
+/**/
+    3803,
+/**/
+    3802,
+/**/
+    3801,
+/**/
+    3800,
+/**/
+    3799,
+/**/
+    3798,
+/**/
+    3797,
+/**/
+    3796,
+/**/
+    3795,
+/**/
+    3794,
+/**/
+    3793,
+/**/
+    3792,
+/**/
+    3791,
+/**/
+    3790,
+/**/
+    3789,
+/**/
+    3788,
+/**/
+    3787,
+/**/
+    3786,
+/**/
+    3785,
+/**/
+    3784,
+/**/
+    3783,
+/**/
+    3782,
+/**/
+    3781,
+/**/
+    3780,
+/**/
+    3779,
+/**/
+    3778,
+/**/
+    3777,
+/**/
+    3776,
+/**/
+    3775,
+/**/
+    3774,
+/**/
+    3773,
+/**/
+    3772,
+/**/
+    3771,
+/**/
+    3770,
+/**/
+    3769,
+/**/
+    3768,
+/**/
+    3767,
+/**/
+    3766,
+/**/
+    3765,
+/**/
+    3764,
+/**/
+    3763,
+/**/
+    3762,
+/**/
+    3761,
+/**/
+    3760,
+/**/
+    3759,
+/**/
+    3758,
+/**/
+    3757,
+/**/
+    3756,
 /**/
     3755,
 /**/
@@ -8409,7 +9640,7 @@ list_in_columns(char_u **items, int size, int current)
     // The rightmost column doesn't need a separator.
     // Sacrifice it to fit in one more column if possible.
     ncol = (int) (Columns + 1) / width;
-    nrow = item_count / ncol + (item_count % ncol ? 1 : 0);
+    nrow = item_count / ncol + ((item_count % ncol) ? 1 : 0);
 
     // "i" counts columns then rows.  "idx" counts rows then columns.
     for (i = 0; !got_int && i < nrow * ncol; ++i)
